"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FeatureFlagActionProvider = void 0;
const tslib_1 = require("tslib");
// Copyright (c) 2023 Sourcefuse Technologies
//
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT
const core_1 = require("@loopback/core");
const lodash_1 = require("lodash");
const loopback4_authentication_1 = require("loopback4-authentication");
const keys_1 = require("../keys");
const rest_1 = require("@loopback/rest");
const services_1 = require("../services");
let FeatureFlagActionProvider = class FeatureFlagActionProvider {
    constructor(getMetadata, user, handlerService) {
        this.getMetadata = getMetadata;
        this.user = user;
        this.handlerService = handlerService;
    }
    value() {
        return () => this.action();
    }
    async action() {
        var _a, _b, _c, _d, _e;
        const metadata = await this.getMetadata();
        if (metadata && this.user) {
            const featureToCheck = [metadata.featureKey];
            if (featureToCheck.length && featureToCheck[0] === '*') {
                return true; // skip the check and always return true
            }
            const disabledFeatures = (_b = (_a = this.user) === null || _a === void 0 ? void 0 : _a.disabledFeatures) !== null && _b !== void 0 ? _b : [];
            if (!disabledFeatures) {
                throw new rest_1.HttpErrors.InternalServerError('List of disabled features not passed to user');
            }
            const isAllowed = (0, lodash_1.intersection)(featureToCheck, disabledFeatures).length <= 0;
            if (!isAllowed) {
                return false;
            }
            else {
                // check for handler
                const handler = (_c = metadata.options) === null || _c === void 0 ? void 0 : _c.handler;
                if (handler) {
                    return (_e = (_d = this.handlerService) === null || _d === void 0 ? void 0 : _d.handle(metadata, this.user)) !== null && _e !== void 0 ? _e : true;
                }
                return true;
            }
        }
        else {
            return true; // not mandatory to apply feature check over all API
        }
    }
};
FeatureFlagActionProvider = tslib_1.__decorate([
    tslib_1.__param(0, core_1.inject.getter(keys_1.StrategyBindings.METADATA)),
    tslib_1.__param(1, (0, core_1.inject)(loopback4_authentication_1.AuthenticationBindings.CURRENT_USER)),
    tslib_1.__param(2, (0, core_1.service)(services_1.FeatureHandlerService)),
    tslib_1.__metadata("design:paramtypes", [Function, Object, services_1.FeatureHandlerService])
], FeatureFlagActionProvider);
exports.FeatureFlagActionProvider = FeatureFlagActionProvider;
//# sourceMappingURL=feature-flag-action.provider.js.map